function [coeff,fitted,stats] = ols(y,x,addconstant)
% function [coeff,fitted,stats] = ols(y,x,addconstant) runs ols regression 
%           of y = beta_0 + beta_1 * x + error
% INPUTS:   y(T*Ny), x(T*Nx): data
%           addconstant: dummy for if we want to add a constant to x to
%                   calculate intercept. 
%                   = 0 if x already contains a column of ones
%                   = 1 (default) if x doesn't have a column of ones
% OUTPUT:   coeff:  a structure containing relevant coefficients
%                   coeff.b0: intercept
%                   coeff.b1: slope
%                   coeff.b: write b0 and b1 in a matrix form
%           fitted: a structure containing fitted values and residuals
%                   fitted.yhat: fitted values
%                   fitted.e: residuals
%           stats:  a structure of various statistics
%                   stats.s: standard error
%                   stats.std_b: standard error for parameters;
%                   stats.t: t stats for tests: b_i = 0;
%                   stats.p: p values for the test H0: b_i = 0;
%                   stats.R2: R squared
%                   stats.R2_adj: adjusted R squared
%                   stats.Hessian: variance-covariance matrix for b
%                   stats.sse: sum of squared residuals
% Cynthia Wu @ Chicago Booth
% Last update: 9/21/2011


[N,k] = size(x);
if nargin <= 2 || addconstant == 1
    N = length(y);
    x = [ones(N,1),x];
end
para = x\y;     % this is more efficient in terms of computation, you can also try para = inv(x'*x)*(x'*y)
coeff.b0 = para(1,:)';
coeff.b1 = para(2:end,:)';
coeff.b = para;

if nargout > 1
    fitted.yhat = x*para;
    e = y - x*para;
    Omega = e'*e/(N-k-1);
    coeff.Omega = Omega;
    s= sqrt(diag(Omega));
    fitted.e = e;
    %temp = regstats(y,x(:,2:end),'linear');
    %fitted.e_student = temp.studres;
    if nargout > 2
        H = kron(Omega,inv(x'*x));      % this way we can calculate variance-covariance matrix for b when there are multiple y. For single y, you can try Omega/(x'*x)
        stats.Hessian = H;
        std_b = reshape(sqrt(diag(H)),size(para));       
        t = para./std_b;
        stats.s = s;
        stats.std_b = std_b;
        stats.t = t;
        stats.p = 2*(1-tcdf(abs(t),N-k-1));
        stats.R2 = 1 - diag(e'*e) ./ diag((y-repmat(mean(y),N,1))'*(y-repmat(mean(y),N,1)));
        stats.R2_adj = 1 - (diag(e'*e)/(N-k-1)) ./ (diag((y-repmat(mean(y),N,1))'*(y-repmat(mean(y),N,1)))/(N-1));
        stats.sse = e'*e; 
        d = duplication(size(Omega,1));
        T = length(x);
        Hv = inv(T/2*d'*kron(inv(Omega),inv(Omega))*d);
        std_Omega = veclt(sqrt(diag(Hv)));
        stats.std_Omega = std_Omega;
    end
end